import { Plugin, ButtonView } from 'ckeditor5';

import ckeditor5Icon from '../theme/icons/ckeditor.svg';

import { asBlob } from 'html-docx-js-typescript';
import { saveAs } from 'file-saver';

import Temml from 'temml';

export default class Export extends Plugin {
	public static get pluginName() {
		return 'Export' as const;
	}

	public init(): void {
		const editor = this.editor;
		const t = editor.t;
		const model = editor.model;

		// Add the "exportButton" to feature components.
		editor.ui.componentFactory.add( 'exportButton', locale => {
			const view = new ButtonView( locale );

			view.set( {
				label: t( 'Export' ),
				icon: ckeditor5Icon,
				tooltip: true
			} );

			// // Insert a text into the editor after clicking the button.
			// this.listenTo( view, 'execute', () => {
			// 	model.change( writer => {
			// 		const textNode = writer.createText( 'Hello CKEditor 5!' );

			// 		model.insertContent( textNode );
			// 	} );

			// 	editor.editing.view.focus();
			// } );

			// Insert a text into the editor after clicking the button.
			// this.listenTo( view, 'execute', async () => {
			// 	asBlob(this.editor.getData()).then((data: any) => {
			// 		saveAs(data, 'file.docx') // save as docx file
			// 	}) // asBlob() return Promise<Blob|Buffer>

			// 	editor.editing.view.focus();
			// } );

			// POST fetch docx and download
			this.listenTo( view, 'execute', async () => {
				const parser = new DOMParser();
				const doc = parser.parseFromString(this.editor.getData(), "text/html");

				// pre-process mathlive script to html MathML tag with the help of temml
				for (const mathliveElement of doc.querySelectorAll('script[type="math/tex"]')) {
					const temmlWrapperElement = document.createElement('span');
					const temmlMathMLString = Temml.render(mathliveElement.textContent ?? "", temmlWrapperElement);
					mathliveElement.replaceWith(temmlWrapperElement);
				}

				// pre-process for table
				for (const table of doc.querySelectorAll('table')) {
					// pre-process table for border and centering and inner border
					table.style.borderCollapse = 'collapse';
					table.style.borderSpacing = '0';
					table.style.border = '1px solid #b3b3b3';
					table.style.margin = 'auto';
					// pre-process table for padding
					for (const td of table.querySelectorAll('td')) {
						td.style.padding = '8px';
					}
					// pre-process table for inner border
					for (const tr of table.querySelectorAll('tr')) {
						for (const td of tr.querySelectorAll('td')) {
							td.style.border = '1px solid #b3b3b3';
						}
					}
					// pre-process table for header background color rgba(0,0,0,.05) to hex #f2f2f2
					// and it's own border
					// and padding
					const ths = table.querySelectorAll('th');
					for (const th of ths) {
						th.style.backgroundColor = '#f2f2f2';
						th.style.border = '1px solid #b3b3b3';
						th.style.padding = '8px';
					}
				}

				console.log(doc.body.innerHTML);

				fetch('http://localhost:8080/aspose/html-to-word', {
					method: 'POST',
					body: doc.body.innerHTML,
					headers: {
						'Content-Type': 'text/plain'
					}
				})
				.then(response => response.blob())
				.then(blob => {
					saveAs(blob, 'stou.docx') // save as docx file
				})
				.catch(error => {
					console.error('Error:', error);
				});
				editor.editing.view.focus();
			} );

			return view;
		} );
	}
}
